@@ -1,6 +1,4 @@
-# file openpyxl/reader/excel.py
-
-# Copyright (c) 2010-2011 openpyxl
+# Copyright (c) 2010-2014 openpyxl
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,28 +25,34 @@
 
 # Python stdlib imports
 from zipfile import ZipFile, ZIP_DEFLATED, BadZipfile
+from sys import exc_info
+import warnings
+
+# compatibility imports
+
+from openpyxl.shared.compat import file
 
 # package imports
 from openpyxl.shared.exc import OpenModeError, InvalidFileException
 from openpyxl.shared.ooxml import (ARC_SHARED_STRINGS, ARC_CORE, ARC_WORKBOOK,
-                                   PACKAGE_WORKSHEETS, ARC_STYLE, ARC_THEME)
-from openpyxl.shared.compat import unicode, file
+                                   PACKAGE_WORKSHEETS, ARC_STYLE, ARC_THEME,
+                                   ARC_CONTENT_TYPES)
+from openpyxl.shared.compat import unicode, file, BytesIO, StringIO
 from openpyxl.workbook import Workbook, DocumentProperties
 from openpyxl.reader.strings import read_string_table
 from openpyxl.reader.style import read_style_table
 from openpyxl.reader.workbook import (read_sheets_titles, read_named_ranges,
-        read_properties_core, read_excel_base_date, get_sheet_ids)
+        read_properties_core, read_excel_base_date, get_sheet_ids,
+        read_content_types)
 from openpyxl.reader.worksheet import read_worksheet
-from openpyxl.reader.iter_worksheet import unpack_worksheet
+from openpyxl.reader.comments import read_comments, get_comments_file
 # Use exc_info for Python 2 compatibility with "except Exception[,/ as] e"
-from sys import exc_info
 
-try:
-    # Python 2
-    unicode
-except NameError:
-    # Python 3
-    unicode = str
+
+VALID_WORKSHEET = "application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml"
+VALID_CHARTSHEET = "application/vnd.openxmlformats-officedocument.spreadsheetml.chartsheet+xml"
+WORK_OR_CHART_TYPE = [VALID_WORKSHEET, VALID_CHARTSHEET]
+
 
 CENTRAL_DIRECTORY_SIGNATURE = '\x50\x4b\x05\x06'
 
@@ -56,10 +60,11 @@ def repair_central_directory(zipFile, is_file_instance):
     ''' trims trailing data from the central directory
     code taken from http://stackoverflow.com/a/7457686/570216, courtesy of Uri Cohen
     '''
-    from StringIO import StringIO
 
     f = zipFile if is_file_instance else open(zipFile, 'r+b')
-    data = f.read().decode("utf-8")
+    data = f.read()
+    if hasattr(data, "decode"):
+        data = data.decode("utf-8")
     pos = data.find(CENTRAL_DIRECTORY_SIGNATURE)  # End of central directory signature
     if (pos > 0):
         sio = StringIO(data)
@@ -72,7 +77,7 @@ def repair_central_directory(zipFile, is_file_instance):
     return f
 
 
-def load_workbook(filename, use_iterators=False):
+def load_workbook(filename, use_iterators=False, keep_vba=False, guess_types=True, data_only=False):
     """Open the given filename and return the workbook
 
     :param filename: the path to open or a file-like object
@@ -81,6 +86,12 @@ def load_workbook(filename, use_iterators=False):
     :param use_iterators: use lazy load for cells
     :type use_iterators: bool
 
+    :param keep_vba: preseve vba content (this does NOT mean you can use it)
+    :type keep_vba: bool
+
+    :param guess_types: guess cell content type and do not read it from the file
+    :type guess_types: bool
+
     :rtype: :class:`openpyxl.workbook.Workbook`
 
     .. note::
@@ -90,14 +101,7 @@ def load_workbook(filename, use_iterators=False):
 
     """
 
-    try:
-        # Python 2
-        is_file_instance = isinstance(filename, file)
-    except NameError:
-        # Python 3
-        from io import BufferedReader
-        is_file_instance = isinstance(filename, BufferedReader)
-
+    is_file_instance = isinstance(filename, file)
 
     if is_file_instance:
         # fileobject must have been opened with 'rb' flag
@@ -117,27 +121,39 @@ def load_workbook(filename, use_iterators=False):
     except (BadZipfile, RuntimeError, IOError, ValueError):
         e = exc_info()[1]
         raise InvalidFileException(unicode(e))
-    wb = Workbook()
+    wb = Workbook(guess_types=guess_types, data_only=data_only)
 
     if use_iterators:
         wb._set_optimized_read()
+        if not guess_types:
+            warnings.warn('please note that data types are not guessed '
+                          'when using iterator reader, so you do not need '
+                          'to use guess_types=False')
 
     try:
-        _load_workbook(wb, archive, filename, use_iterators)
+        _load_workbook(wb, archive, filename, use_iterators, keep_vba)
     except KeyError:
         e = exc_info()[1]
         raise InvalidFileException(unicode(e))
 
-    archive.close()
+    if not keep_vba:
+        archive.close()
     return wb
 
-def _load_workbook(wb, archive, filename, use_iterators):
+
+def _load_workbook(wb, archive, filename, use_iterators, keep_vba):
 
     valid_files = archive.namelist()
 
+    # If are going to preserve the vba then attach the archive to the
+    # workbook so that is available for the save.
+    if keep_vba:
+        wb.vba_archive = archive
+
     # get workbook-level information
     try:
         wb.properties = read_properties_core(archive.read(ARC_CORE))
+        wb.read_workbook_settings(archive.read(ARC_WORKBOOK))
     except KeyError:
         wb.properties = DocumentProperties()
 
@@ -150,14 +166,19 @@ def _load_workbook(wb, archive, filename, use_iterators):
     except KeyError:
         assert wb.loaded_theme == None, "even though the theme information is missing there is a theme object ?"
 
-    style_table = read_style_table(archive.read(ARC_STYLE))
+    style_properties = read_style_table(archive.read(ARC_STYLE))
+    style_table = style_properties.pop('table')
+    wb.style_properties = style_properties
 
     wb.properties.excel_base_date = read_excel_base_date(xml_source=archive.read(ARC_WORKBOOK))
 
     # get worksheets
     wb.worksheets = []  # remove preset worksheet
+    content_types = read_content_types(archive.read(ARC_CONTENT_TYPES))
+    sheet_types = [(sheet, contyp) for sheet, contyp in content_types if contyp in WORK_OR_CHART_TYPE]
     sheet_names = read_sheets_titles(archive.read(ARC_WORKBOOK))
-    for i, sheet_name in enumerate(sheet_names):
+    worksheet_names = [worksheet for worksheet, sheet_type in zip(sheet_names, sheet_types) if sheet_type[1] == VALID_WORKSHEET]
+    for i, sheet_name in enumerate(worksheet_names):
 
         sheet_codename = 'sheet%d.xml' % (i + 1)
         worksheet_path = '%s/%s' % (PACKAGE_WORKSHEETS, sheet_codename)
@@ -166,10 +187,21 @@ def _load_workbook(wb, archive, filename, use_iterators):
             continue
 
         if not use_iterators:
-            new_ws = read_worksheet(archive.read(worksheet_path), wb, sheet_name, string_table, style_table)
+            new_ws = read_worksheet(archive.read(worksheet_path), wb,
+                                    sheet_name, string_table, style_table,
+                                    color_index=style_properties['color_index'],
+                                    keep_vba=keep_vba)
         else:
-            xml_source = unpack_worksheet(archive, worksheet_path)
-            new_ws = read_worksheet(xml_source, wb, sheet_name, string_table, style_table, filename, sheet_codename)
+            new_ws = read_worksheet(None, wb, sheet_name, string_table,
+                                    style_table,
+                                    color_index=style_properties['color_index'],
+                                    workbook_name=filename,
+                                    sheet_codename=sheet_codename)
         wb.add_sheet(new_ws, index=i)
 
+        # load comments into the worksheet cells
+        comments_file = get_comments_file(sheet_codename, archive, valid_files)
+        if comments_file is not None:
+            read_comments(new_ws, archive.read(comments_file))
+
     wb._named_ranges = read_named_ranges(archive.read(ARC_WORKBOOK), wb)
